/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.maven.plugins.license;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.model.License;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.ProjectBuildingException;

/**
 * Maven goal for downloading the license files of all the current project's dependencies.
 * 
 * @phase generate-resources
 * @goal download-licenses
 * @requiresDependencyResolution test
 * 
 * @author Paul Gier
 * @version $Revision: 321 $
 */
public class DownloadLicensesMojo extends AbstractMojo
{

   /**
    * The Maven Project Object
    *
    * @parameter expression="${project}"
    * @readonly
    */
   protected MavenProject project;

   /**
    * Used to build a maven projects from artifacts in the remote repository.
    * 
    * @parameter expression="${component.org.apache.maven.project.MavenProjectBuilder}"
    * @required
    * @readonly
    */
   protected org.apache.maven.project.MavenProjectBuilder projectBuilder;

   /**
    * Location of the local repository.
    * 
    * @parameter expression="${localRepository}"
    * @readonly
    * @required
    */
   protected org.apache.maven.artifact.repository.ArtifactRepository localRepository;

   /**
    * List of Remote Repositories used by the resolver
    * 
    * @parameter expression="${project.remoteArtifactRepositories}"
    * @readonly
    * @required
    */
   protected java.util.List remoteRepositories;

   /**
    * The directory to which the dependency licenses should be written.
    * 
    * @parameter default-value="${project.build.directory}/licenses"
    */
   private File outputDirectory;

   /**
    * File contains a mapping between each dependency and it's license information.
    * 
    * @parameter default-value="${project.build.directory}/licenses.xml"
    */
   private File licenseSummaryFile;

   /**
    * Don't show warnings about bad or missing license files.
    * 
    * @parameter default-value="false"
    */
   private boolean quiet;

   public void execute() throws MojoExecutionException
   {
      getLog().info("Downloading licenses");
      
      if (!outputDirectory.exists())
      {
         outputDirectory.mkdirs();
      }
      
      if (!licenseSummaryFile.getParentFile().exists())
      {
         licenseSummaryFile.getParentFile().mkdirs();
      }

      PrintWriter licensesFileWriter = null;
      try
      {
         licensesFileWriter = new PrintWriter( licenseSummaryFile );
         licensesFileWriter.println("<!-- This file contains license information for the dependencies of "
               + project.getArtifact().getDependencyConflictId() + " -->");
         licensesFileWriter.println("<dependencies>");
      }
      catch (IOException e)
      {
         throw new MojoExecutionException("Unable to open licenseInfo file ", e);
      }

      Set depArtifacts = project.getDependencyArtifacts();
      Iterator depArtIter = depArtifacts.iterator();
      while (depArtIter.hasNext())
      {
         Artifact depArt = (Artifact) depArtIter.next();
         MavenProject depProject = null;
         try
         {
            depProject = projectBuilder.buildFromRepository(depArt, remoteRepositories, localRepository);
         }
         catch (ProjectBuildingException e)
         {
            throw new MojoExecutionException( "Unable to build project: " + depArt.getDependencyConflictId(), e );
         }
         licensesFileWriter.println("  <dependency>");
         licensesFileWriter.println("    <groupId>" + depProject.getGroupId() + "</groupId>");
         licensesFileWriter.println("    <artifactId>" + depProject.getArtifactId() + "</artifactId>");
         licensesFileWriter.println("    <version>" + depProject.getVersion() + "</version>");

         List licenses = depProject.getLicenses();

         if (licenses.isEmpty())
         {
            licensesFileWriter.println("    <!-- No license information found -->");
            licensesFileWriter.println("  </dependency>");
            if (!quiet)
            {
               getLog().warn("POM for dependency " + depProject.getId() + " does not contain license information.");
            }
            continue;
         }

         licensesFileWriter.println("    <licenses>");

         Iterator licenseIter = licenses.iterator();
         while (licenseIter.hasNext())
         {
            License license = (License) licenseIter.next();

            licensesFileWriter.println("      <license>");
            licensesFileWriter.println("        <name>" + license.getName() + "</name>");
            licensesFileWriter.println("        <url>" + license.getUrl() + "</url>");
            licensesFileWriter.println("      </license>");

            URL licenseUrl = null;

            try
            {

               String licenseFileName = this.getLicenseFileName(license);

               File licenseOutputFile = new File(outputDirectory, licenseFileName);
               if (licenseOutputFile.exists())
               {
                  continue;
               }

               licenseUrl = new URL(license.getUrl());
               InputStream licenseInputStream = licenseUrl.openStream();
               FileOutputStream fileOut = new FileOutputStream(licenseOutputFile);
               copyStream(licenseInputStream, fileOut);
               licenseInputStream.close();
               fileOut.close();
            }
            catch (MalformedURLException e)
            {
               if (!quiet)
               {
                  getLog().warn("POM for dependency " + depProject.getId() + " has an invalid license URL: "
                             + license.getUrl());
               }
            }
            catch (FileNotFoundException e)
            {
               if (!quiet)
               {
                  getLog().warn("POM for dependency " + depProject.getId() + " has a license URL that returns file not found: "
                              + license.getUrl());
               }
            }
            catch (IOException e)
            {
               getLog().warn(e);
            }

         }
         licensesFileWriter.println("    </licenses>");
         licensesFileWriter.println("  </dependency>");
      }
      licensesFileWriter.println("</dependencies>");
      licensesFileWriter.close();
   }

   /**
    * Copy data from one stream to another.
    * 
    * @param inStream
    * @param outStream
    * @throws IOException
    */
   private static void copyStream(InputStream inStream, OutputStream outStream) throws IOException
   {
      byte[] buf = new byte[1024];
      int len;
      while ((len = inStream.read(buf)) > 0)
      {
         outStream.write(buf, 0, len);
      }
   }

   /**
    * Tries to determine what the name of the downloaded license file should be
    * based on the information in the license object.
    * 
    * @param license
    * @return
    */
   private String getLicenseFileName(License license) throws MalformedURLException
   {
      URL licenseUrl = new URL(license.getUrl());
      File licenseUrlFile = new File(licenseUrl.getPath());
      String licenseFileName = licenseUrlFile.getName();

      if (license.getName() != null)
      {
         licenseFileName = license.getName() + " - " + licenseUrlFile.getName();
      }

      // Check if the file has a valid file extention
      final String DEFAULT_EXTENSION = ".txt";
      int extensionIndex = licenseFileName.lastIndexOf(".");
      if (extensionIndex == -1 || extensionIndex > (licenseFileName.length() - 3))
      {
         // This means it isn't a valid file extension, so append the default
         licenseFileName = licenseFileName + DEFAULT_EXTENSION;
      }

      return licenseFileName;
   }
}
